local function makeRectEntity(x, y, width, height)
  local rect = {
    --1
    x,
    y,
    --2
    x + width,
    y,
    --3
    x + width,
    y + height,
    --4
    x,
    y + height
  }
  return rect
end


local function updateShape(self, dt)
  local shape = self.shape

  --update pivot
  self.pivot.x = self.pivot.x + self.currSpeed * dt * math.cos(self.vector.direction)
  self.pivot.y = self.pivot.y + self.currSpeed * dt * math.sin(self.vector.direction)

  --update points
  for i = 1, #shape - 1, 2 do
    --x,y
    shape[i] = shape[i] + self.currSpeed * dt * math.cos(self.vector.direction)
    shape[i + 1] = shape[i + 1] + self.currSpeed * dt * math.sin(self.vector.direction)
  end

  --debug
  print("direction: " .. string.format("%.2f",self.vector.direction))
  print("pivot X,Y: (" .. string.format("%2d",self.pivot.x) .. ", " .. string.format("%2d",self.pivot.y) .. ")")

  --add rotation
  local step = 0
  --- -[[
  for i = 1, #shape - 1, 2 do --check x,y pairs for the 4 points of the rectangle (8 total values)
    --distance from current point to pivot
    local distance = math.sqrt(math.pow(shape[i] - self.pivot.x, 2) + math.pow(shape[i + 1] - self.pivot.y, 2))
    --angle from pivot to point
    local angle = math.atan2(shape[i + 1] - self.pivot.y, shape[i] - self.pivot.x) + self.vector.direction
    --debug
    print("X" .. i - step .. ",Y" .. i - step .. " Point angle:" .. string.format("%.2f",angle)) step = step + 1

    --x,y
    shape[i] = self.pivot.x + distance * math.cos(angle)
    shape[i + 1] = self.pivot.y + distance * math.sin(angle)
  end
  --]]
end

local function isBoxCollision(rect1, rect2)
  local isCollision = false
  if rect1.x < rect2.x + rect2.width and
     rect1.x + rect1.width > rect2.x and
     rect1.y < rect2.y + rect2.height and
     rect1.y + rect1.height > rect2.y then
    isCollision = true
  end
  return isCollision
end

--Methods
helpers =
{
  makeRectEntity = makeRectEntity,
  updateShape = updateShape,
  isBoxCollision = isBoxCollision
}