require 'helpers'
--vars--
local tankColor = { 145, 140, 100, 255 }
-- -- --



local function _rotateShape()
end

local function _decelerate(tank, dt)
  local newSpeed
  if tank.currSpeed > 0 then
    newSpeed = tank.currSpeed - tank.deceleration * dt
    tank.currSpeed = newSpeed > 0 and newSpeed or 0
  else
    newSpeed = tank.currSpeed + tank.deceleration * dt
    tank.currSpeed = newSpeed < 0 and newSpeed or 0
  end
end

local function updateSpeed(self, dt)
  if self.vector.magnitude ~= 0 then

    --if we thrust in the same direction of our movement, then just increase the speed
    if (self.vector.magnitude > 0 and self.currSpeed >= 0) or (self.vector.magnitude < 0 and self.currSpeed <= 0) then

      local newSpeed = self.currSpeed + (self.acceleration * dt) * self.vector.magnitude
      --set new speed and cap based on maxSpeed
      self.currSpeed = math.abs(newSpeed) < self.maxSpeed and newSpeed or self.maxSpeed * self.vector.magnitude

    else --reverse direction input case
      _decelerate(self, dt)
    end
  else -- no player input case, slow down
    _decelerate(self, dt)
  end
end

local function updatePos(self, dt)

  self:updateShape(dt)

end

local function update(self, dt)
  self:updateSpeed(dt)
  self:updatePos(dt)

  if love.keyboard.isDown('a') then self.vector.direction = (self.vector.direction - self.turningSpeed * dt) % (math.pi * 2)
  elseif love.keyboard.isDown('d') then self.vector.direction = (self.vector.direction + self.turningSpeed * dt) % (math.pi * 2)
  end
end

local function draw(self)
  love.graphics.setColor(self.tankColor)
  love.graphics.polygon("fill", self.shape)

  --pivot
  love.graphics.setColor(255,0,0,255)
  love.graphics.circle("fill",self.pivot.x,self.pivot.y,5,5)
  local endX = self.pivot.x+100*math.cos(self.vector.direction)
  local endY =self.pivot.y +100*math.sin(self.vector.direction)
  love.graphics.line(self.pivot.x,self.pivot.y,endX,endY)
end


local function keypressed(self, key)

  if 'w' == key then self.vector.magnitude = 1
  elseif 's' == key then self.vector.magnitude = -1
  end
end

local function keyreleased(self, key)

  if 'w' == key then self.vector.magnitude = 0
  end
  if 's' == key then self.vector.magnitude = 0
  end
end

local function makeTank(x, y)
  local tankW = 90
  local tankH = 75
  local newTank = {}
  newTank.shape = helpers.makeRectEntity(x - tankW / 2, y - tankH / 2, tankW, tankH)
  newTank.type = "tank"
  newTank.width = 90
  newTank.height = 75
  newTank.pivot = { x = x, y = y }
  newTank.vector = { direction = 0, magnitude = 0 }
  newTank.turningSpeed = math.pi / 5.5
  newTank.acceleration = 25
  newTank.deceleration = 55
  newTank.currSpeed = 0
  newTank.maxSpeed = 90
  newTank.tankColor = tankColor
  newTank.update = update
  newTank.updateSpeed = updateSpeed
  newTank.updateShape = helpers.updateShape
  newTank.updatePos = updatePos
  newTank.draw = draw
  newTank.keypressed = keypressed
  newTank.keyreleased = keyreleased
  return newTank
end



--Methods
tank =
{
  makeTank = makeTank
}